const test = require('blue-tape');
const { output, sippUac } = require('./sipp')('test2_testbed');
const Uas = require('./scripts/uas');
//const debug = require('debug')('test:drachtio-srf');

process.on('unhandledRejection', (reason, p) => {
  console.log('Unhandled Rejection at: Promise', p, 'reason:', reason);
});

test('UAS', (t) => {
  t.timeoutAfter(60000);

  let uas = new Uas();

  Promise.resolve()

    // return 503
    .then(() => {
      return uas.reject(503);
    })
    .then((uas) => {
      return sippUac('uac-expect-503.xml');
    })
    .then(() => {
      t.pass('uas reject INVITE with 503');
      return uas.disconnect();
    }).

    //Srf#createUAS returns a Promise
    then(() => {
      uas = new Uas();
      return uas.accept();
    })
    .then((uas) => {
      return sippUac('uac.xml');
    })
    .then(() => {
      return t.pass('Srf#createUAS returns a promise');
    })

    //Srf#createUAS ca accept a callback
    .then(() => {
      uas = new Uas();
      return uas.acceptCb();
    })
    .then((uas) => {
      return sippUac('uac.xml');
    })
    .then(() => {
      return t.pass('Srf#createUAS accepts a callback');
    })

    //Srf#createUAS creates a dialog on 200 OK, not ACK
    .then(() => {
      uas = new Uas();
      return uas.accept();
    })
    .then((uas) => {
      uas
        .on('connected', (uas) => {
          t.pass('uas dialog created when 200 OK sent');
          uas.on('destroy', (msg, reason) => {
            if (reason === 'ACK timeout') {
              t.pass('uas dialog destroyed with ACK timeout after 32s');
            }
          });
        })
        .on('error', (err) => {
          t.error(err);
        });
      return sippUac('uac-drop-all-200.xml');
    })
    .then(() => {
      t.pass('UAS tests completed');
      return new Promise((resolve, reject) => {
        setTimeout(() => {
          uas.disconnect();
          resolve();
        }, 1000);
      });
    })
    .then(() => {
      return t.end();
    })
    .catch((err) => {
      if (uas) uas.disconnect();
      console.log(`error received: ${err}`);
      console.log(output());
      t.error(err);
    });
});
